/* bandmtrx.c
 * Matrixtyp Bandmatrix
 * erstellt: Fre Nov  5 15:04:28 CET 1999
 * Autoren:  Mirko Belickin und Joachim Wagner
 */

#include "matrix.h"
#include "namen.h"
#include "bandmtrx.h"
#include "nmmatrix.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>


/* externes Klassenobjekt einbinden */

extern matrix nm_matrix;


/* Deklaration der Objekt-Methoden */

matrix * bandmatrix_laden(FILE *datei, void *vater);
int bandmatrix_speichern(matrix *m, FILE *datei);
matrix * bandmatrix_kreieren(void *schablone, void *vater);
int bandmatrix_freigeben(matrix *m, void *vater);
double bandmatrix_amIndex(matrix *m, int zeile, int spalte);
void bandmatrix_setzen(matrix *m, int zeile, int spalte, double d);
int bandmatrix_breite(matrix *m);
int bandmatrix_hoehe(matrix *m);
char *bandmatrix_name(matrix *m);
matrix * bandmatrix_imVektor(matrix *m, int vektor);
int bandmatrix_bandbreite(matrix *m);


/* Globales Klassenobjekt */

matrixFunktionen band_matrixFunktionen = {
    matrix_ausgeben,
    bandmatrix_speichern,
    bandmatrix_laden,
    bandmatrix_freigeben,
    bandmatrix_kreieren,
    bandmatrix_amIndex,
    bandmatrix_setzen,
    bandmatrix_breite,
    bandmatrix_hoehe,
    bandmatrix_name,
    matrix_klasse,
    nmmatrix_produkt,
    bandmatrix_imVektor,
    bandmatrix_bandbreite
};

matrix band_matrix = {          /* das Klassenobjekt zur Klasse BAND_MATRIX */ 
    &band_matrixFunktionen,     
    NULL                      /* Klassenobjekt hat keine Daten */
};


/* private Methoden */

static bandmatrixdaten *get_bandmatrixdaten(matrix *m);
static matrix * bandmatrix_pkreieren(void *schablone, void *vater);
static int lese_datum(FILE *datei, char *format, char * prefix, void *pdatum);


/* ------------------------------------------------------------------------ 
 * Band-Matrix Implementation
 */ 


matrix * bandmatrix_laden(FILE *datei, void *vater)
{
  bandmatrixschablone schablone;
  int i;
  matrix *m;
  bandmatrixdaten *pm;

  schablone.name = matrix_kopfzeile(datei, S_BAND_MATRIX);

  /* Abmasse und Bandbreite der Matrix feststellen */

  lese_datum(datei, "%d", "n =", &(schablone.n));
  lese_datum(datei, "%d", "m =", &(schablone.m));

  /* Rohmatrix ohne Diagonal-Vektoren kreieren */

  m = bandmatrix_pkreieren(&schablone, vater);

  /* Eintraege aus der Datei lesen */

  pm = get_bandmatrixdaten(m);

  for (i = 0; i < pm->m * 2 + 1; i++)
    pm->vektor[i] = nm_matrix.aktion->laden(datei, vater);

  return m;
}


static int lese_datum(FILE *datei, char *format, char * prefix, void *pdatum)
{
  char eingabe[BUFSIZ];
  if (!fgets(eingabe,BUFSIZ,datei)) {
    fprintf(stderr,"Zeile %s nicht gefunden - EOF.\n", prefix);
    exit(1);
  }
  if (strncmp(eingabe, prefix,
               strlen( prefix))!=0) {
    fprintf(stderr,"%s... erwartet!\n", prefix);
    exit(1);
  }
  sscanf(&(eingabe[strlen(prefix)]), format, pdatum);
                               /* erstmal ohne Fehlerkontrolle */
  return 0;
}


int bandmatrix_speichern(matrix *m, FILE *datei)
{
  bandmatrixdaten *pm;
  int i;
  pm = get_bandmatrixdaten(m);
  fprintf(datei,S_BAND_MATRIX ": %s\n", pm->name);
  fprintf(datei,"n = %d\n", pm->n);
  fprintf(datei,"m = %d\n", pm->m);
  for (i = 0; i < pm->m * 2 + 1; i++)
    pm->vektor[i]->aktion->speichern(pm->vektor[i], datei);
  return 0;
}


static matrix * bandmatrix_pkreieren(void *schablone, void *vater)
{
  bandmatrixschablone *s = (bandmatrixschablone *) schablone;
  matrix *m;
  bandmatrixdaten *pm;
  char *name;
  if (!(m = (matrix *)malloc(sizeof(matrix)))) {
    fprintf(stderr,"Nicht genug Speicher fuer neue Matrix (Teil 1)!\n");
    exit(1);          
  }
  m->aktion = band_matrix.aktion;    /* moegliche Aktionen festlegen */
  if (!(m->daten = (bandmatrixdaten *)malloc(sizeof(bandmatrixdaten)))) {
    fprintf(stderr,"Nicht genug Speicher fuer neue Matrix (Teil 2)!\n");
    exit(1);          
  }
  pm = get_bandmatrixdaten(m);

  /* Name der Matrix festlegen */

  name = (s && s->name)?s->name:"unbenannt";
  if (!(pm->name = (char *)malloc( sizeof(char) * (1+strlen(name))))) {
    fprintf(stderr, "Nicht genug Speicher fuer " S_BAND_MATRIX " %s.\n",
                    name);
    exit(1);
  }
  strcpy(pm->name, name);

  pm->n = s?s->n:5;  /* Groesse der Matrix ggf. aus Schablone, sonst 5x5 */
  pm->m = s?s->m:3;  /* Bandbreite m - wenn keine Schablone, dann default 3 */

  if (!(pm->vektor = (matrix **)malloc( sizeof(matrix *)
                                        * (pm->m * 2 + 1)))) {
    fprintf(stderr,"Nicht genug Speicher fuer Matrixeintraege!\n");
    exit(1);          
  }
  return m;
}


matrix * bandmatrix_kreieren(void *schablone, void *vater)
{
  bandmatrixschablone *s = (bandmatrixschablone *) schablone;
  int i;
  matrix *m;
  bandmatrixdaten *pm;
  nmmatrixschablone schablone2; 
  char tempname[80];
  
  /* Rohmatrix ohne Diagonal-Vektoren kreieren */

  m = bandmatrix_pkreieren(schablone, vater);
  pm = get_bandmatrixdaten(m);

  /* Diagonal-Vektor alloziieren */

  schablone2.n = pm->n;  /* Vektor als Spaltenvektor speichern, also */
  schablone2.m = 1;      /* eine n x 1 Matrix */
  schablone2.init = s->init;
  schablone2.name = "Hauptdiagonal-Vektor";
  pm->vektor[0] = nm_matrix.aktion->kreieren(&schablone2, m); 

  /* Nebendiagonal-Vektoren alloziieren */
 
  schablone2.name = tempname;
  for (i=1; i<= pm->m; i++) {
    schablone2.n = pm->n - i;  /* Vektor als Spaltenvektor speichern, also */
    schablone2.m = 1;          /* eine (n-i) x 1 Matrix */
    schablone2.init = s->init;
    /* obere i-te Nebendiagonale erzeugen */
    sprintf(schablone2.name,"oberer %i-ter Nebendiagonal-Vektor", i);
    pm->vektor[i*2-1] = nm_matrix.aktion->kreieren(&schablone2, m); 
    /* untere  i-te Nebendiagonale erzeugen */
    sprintf(schablone2.name,"unterer %i-ter Nebendiagonal-Vektor", i);
    pm->vektor[i*2  ] = nm_matrix.aktion->kreieren(&schablone2, m); 
  }

  return m;
}

int bandmatrix_freigeben(matrix *m, void *vater) 
{
  int i; 
  bandmatrixdaten *pm;
  pm = get_bandmatrixdaten(m);
  for(i=0; i < pm->m * 2 + 1; i++) 
    pm->vektor[i]->aktion->freigeben(pm->vektor[i],m);
  free(pm->name);
  free(pm->vektor);     
  free(pm);
  free(m);
  return 0;              /* immer Erfolg */
}

static bandmatrixdaten *get_bandmatrixdaten(matrix *m) 
{
  if ((!(m->daten))) /* sicher ist sicher ... */
    exit(1);                        /* Fehlermeldung fehlt noch */
  return m->daten;                  /* Daten der Matrix zurueckgeben */
}


#define bandmatrix_pruefeIndex(pm, zeile, spalte)  \
  if ((zeile < 1) ||  \
      (spalte < 1) || \
      (zeile > pm->n) || \
      (spalte > pm->n)) { \
   fprintf(stderr,S_BAND_MATRIX "Ungueltiger Index fuer Matrix: (%i,%i)\n", \
                  zeile, spalte); \
   exit(1); \
  } 


double bandmatrix_amIndex(matrix *m, int zeile, int spalte)
{
  bandmatrixdaten *pm;
  int vnr;
  pm = get_bandmatrixdaten(m);
  bandmatrix_pruefeIndex(pm, zeile, spalte); 

  /* Diagonal-Vektor bestimmen */
  if(zeile==spalte) {
    return pm->vektor[0]->aktion      /*  i-te Komponente im Diagonalvektor */
      ->amIndex(pm->vektor[0], zeile, 1);  /* zurueckgeben */
  }
  if (zeile < spalte) {               /* obere Nebendiagonale */
    vnr=(spalte-zeile)*2-1;
    if (vnr >= (pm->m * 2 + 1))
      return 0.0;
    else
      return pm->vektor[vnr]->aktion
        ->amIndex(pm->vektor[vnr], zeile, 1); 
  } 
  else {                              /* untere Nebendiagonale */
    vnr=(zeile-spalte)*2;
    if (vnr >= (pm->m * 2 + 1))
      return 0.0;
    else
      return pm->vektor[vnr]->aktion
        ->amIndex(pm->vektor[vnr], spalte, 1); 
  }
}

void bandmatrix_setzen(matrix *m, int zeile, int spalte, double d)
{
  bandmatrixdaten *pm;
  int vnr;
  pm = get_bandmatrixdaten(m);
  bandmatrix_pruefeIndex(pm, zeile, spalte); 
 
  /* Diagonal-Vektor bestimmen */
  if(zeile==spalte) {
    pm->vektor[0]->aktion      /*  i-te Komponente im Diagonalvektor */
      ->setzen(pm->vektor[0], zeile, 1 ,d);  /* zurueckgeben */
  }
  if (zeile < spalte) {               /* obere Nebendiagonale */
    vnr=(spalte-zeile)*2-1;
    if ((vnr >= (pm->m * 2 + 1)) && (fabs(d) > EPSILON)) {
      fprintf(stderr,"Ungueltiger Diagonal-Vektor fuer Bandmatrix %s: "
                     "(%i,%i)\n", pm->name, zeile, spalte); 
      exit(1); 
    }
    pm->vektor[vnr]->aktion
      ->setzen(pm->vektor[vnr], zeile, 1, d); 
  } 
  else {                              /* untere Nebendiagonale */
    vnr=(zeile-spalte)*2;
    if ((vnr >= (pm->m * 2 + 1)) && (fabs(d) > EPSILON)) {
      fprintf(stderr,"Ungueltiger Diagonal-Vektor fuer Bandmatrix %s: "
                     "(%i,%i)\n", pm->name, zeile, spalte); 
      exit(1); 
    }
    pm->vektor[vnr]->aktion
      ->setzen(pm->vektor[vnr], spalte, 1 ,d); 
  }
}

int bandmatrix_breite(matrix *m)
{
  bandmatrixdaten *pm;
  pm = get_bandmatrixdaten(m);
  return pm->n;
}

int bandmatrix_hoehe(matrix *m)
{
  bandmatrixdaten *pm;
  pm = get_bandmatrixdaten(m);
  return pm->n;
}

char *bandmatrix_name(matrix *m)
{
  bandmatrixdaten *pm;
  if (!m->daten)
    return S_BAND_MATRIX;
  pm = get_bandmatrixdaten(m);
  return pm->name;
}

int bandmatrix_bandbreite(matrix *m)
{
  bandmatrixdaten *pm;
  pm = get_bandmatrixdaten(m);
  return pm->m;
}

matrix * bandmatrix_imVektor(matrix *m, int vektorNr)
{
  bandmatrixdaten *pm;
  pm = get_bandmatrixdaten(m);
  if (vektorNr >= (pm->m * 2 + 1)) {
    fprintf(stderr,"Ungueltiger Diagonal-Vektor fuer Bandmatrix %s: (%i)\n",
                   pm->name, vektorNr);
    exit(1); 
  }
  return pm->vektor[vektorNr];
}
