/* gbsp.c
 * erzeugt Beispiel-Interpolationsproblem-Dateien
 * erstellt: November 1999
 * Autoren : Mirko Belickin und Joachim Wagner
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "funktion.h"
#include "matrix.h"
#include "nmmatrix.h"
#include "interplt.h"
#include "newtnipl.h"

extern interpolation newton_interpolation;
extern matrix nm_matrix;

extern funktionsbeispiel funktion[];
extern const int funktion_anzahl;


/* Hauptprogramm 
 *
 * erzeugt Beispiele
 */

int main(int argc, char *argv[])
{
  FILE *out = stdout;
  FILE *temp;
  int hilfe = 0;        /* 1 = Hilfe fuer Benutzer anzeigen */   
  int fehler = 0;       /* 1 = Programm falsch aufgerufen */
  int schalter = 1;     /* 0 = es folgen keine Schalter ('--' gefunden) */
  int i, n, m;
  char tempname[80];
  newtoninterpolationsschablone schablone;
  matrix *y, *x;
  nmmatrixschablone schablone2;
  interpolation *polynom;
  
  schablone.stuetzstellenart = 0;
  while ((schalter) && (argc>1) && (*argv[1]=='-')) {/* Schalter bearbeiten */
    switch ((argv[1])[1]) {
      case 'h': hilfe = 1;
                break;
      case 'a': schablone.stuetzstellenart = 1;
                break;
      case 't': schablone.stuetzstellenart = 2;
                break;
      case '-': schalter = 0;
                break;
      default:  fprintf(stderr,"Schalter %s unbekannt.\n", argv[1]);
                fehler = 1;
                break;
    }
    argv[1] = argv[0];
    argv++; 
    argc--;
  }
  
  if (schablone.stuetzstellenart == 0) fehler = 1;
  if ((argc>4) || (argc<3)) fehler = 1;
  if ((hilfe) || (fehler)) {             /* Hilfetext anzeigen */
    fprintf(stderr,"\nBenutzung: "
                   "%s [Schalter] [--] n m [Ausgabedatei]\n\n",
                   argv[0]);
    fprintf(stderr,"Schalter: -h fuer diese Hilfe\n");
    fprintf(stderr,"          -a aequidistante Stuetzstellen\n");
    fprintf(stderr,"          -t Stuetzstellen nach Tschebyscheff-Polynom\n");
    fprintf(stderr,"Anmerkung: Stuetzstellenart muss angegeben werden\n");
    fprintf(stderr," n = Anzahl der Stuetzstellen\n");
    fprintf(stderr," m = Nummer der Beispielfunktion\n");
    fprintf(stderr,"\nMirko Belickin und Joachim Wagner - 09. November 1999\n");
    exit(fehler);
  }
  if (argc>1) {
    n = atoi(argv[1]);
    if (n<1) {
      fprintf(stderr, "n muss mindestens 1 sein!\n");
      exit(1);
    }
  }
  if (argc>2) {
    m = atoi(argv[2]);
    if (m<1) {
      fprintf(stderr, "m muss mindestens 1 sein!\n");
      exit(1);
    }
    if (m>funktion_anzahl) {
      fprintf(stderr, "Funktion Nr. %i nicht definiert.\n", m);
      exit(1);
    }
  }
  if (argc>3) {
    if (!(temp=fopen(argv[3],"wb"))) {
      fprintf(stderr,"Oeffnen der Datei %s fehlgeschlagen.", argv[3]);
      exit(1);
    }
    out = temp;
  }
  
  /* Zum Erstellen eines Interpolationsobjekts wird ein Vektor mit
   * den Stuetzwerten benoetigt. Erstelle also einen Dummy-Vektor: 
   */

  schablone2.n = n + 1;
  schablone2.m = 1;
  sprintf(tempname, "Werte der Beispielfunktion %s", funktion[m-1].name);
  schablone2.name = tempname;
  schablone2.init = 1;          /* mit 0 initialisieren */
  y = nm_matrix.aktion->kreieren(&schablone2, NULL);
 
  /* Interpolationsobjekt erstellen */

  schablone.a = funktion[m-1].a;
  schablone.b = funktion[m-1].b;
  sprintf(tempname, "Beispielfunktion %s mit n = %i", 
                    funktion[m-1].name, n);
  schablone.name = tempname;
  schablone.y = y;

  polynom = newton_interpolation.aktion->kreieren(&schablone, NULL);
  y->aktion->freigeben(y, NULL);  /* Dummy-Vektor freigeben */
  
  /* Vektoren fuer Stuetzstellen und -werte des Interpolationsobjekts */
    
  x = polynom->aktion->imVektor(polynom, 0);
  y = polynom->aktion->imVektor(polynom, 1);
  
  /* Stuetzwerte fuer die gegebenen Stuetzstellen berechnen */

  for (i=1; i <= n + 1; i++)
    y->aktion->setzen(y, i, 1,                       /* Y-Wert der Beispiel- */
      funktion[m-1].wert(x->aktion->amIndex(x, i, 1)));   /*  funktion Nr. m */

  polynom->aktion->speichern(polynom, out);     /* Daten speichern */
  polynom->aktion->freigeben(polynom, NULL);

  if (out != stdout) fclose(out);               /* Dateien ggf. schliessen */
  return 0;
}
