/* koeffizi.c
 * Bestimmt die Koeffizienten des Newtonpolynoms
 * Eingabe:  n + 1 Stuetzpunkte (Format siehe Beispieldateien)
 * Ausgabe:  zugehoeriges Interpolationspolynom in der Newton-Darstellung
 * erstellt: Mon Nov 22 16:39:25 CET 1999
 * Autoren:  Mirko Belickin und Joachim Wagner
 */

#include <stdio.h>
#include <stdlib.h>
#include "interplt.h" 
#include "matrix.h"


/* externe Klassenobjekte einbinden */

extern interpolation newton_interpolation;
extern interpolation allg_interpolation;


/* Hauptprogramm */

int main(int argc, char *argv[])
{
  FILE *in = stdin;     /* Ein-/Ausgabe */
  FILE *out = stdout;
  FILE *temp;
  int retcode;     
  int hilfe = 0;        /* 1 = Hilfe fuer Benutzer anzeigen */   
  int fehler = 0;       /* 1 = Programm falsch aufgerufen */
  int schalter = 1;     /* 0 = es folgen keine Schalter ('--' gefunden) */

  interpolation *polynom;   /* nimmt Parameter und Ergebnis auf */ 
  matrix *c;                /* Koeffizienten-Vektor */

  while ((schalter) && (argc>1) && (*argv[1]=='-')) {/* Schalter bearbeiten */
    switch ((argv[1])[1]) {
      case 'h': hilfe = 1;
                break;
      case '-': schalter = 0;
                break;
      default:  fprintf(stderr,"Schalter %s unbekannt.\n", argv[1]);
                fehler = 1;
                break;
    }
    argv[1] = argv[0];
    argv++; 
    argc--;
  }
  if ((argc>3) || (argc<2)) fehler = 1;
  if ((hilfe) || (fehler)) {             /* Hilfetext anzeigen */
    fprintf(stderr,"\nBenutzung: "
                   "%s [Schalter] [--] Eingabedatei [Ausgabedatei]\n\n",
                   argv[0]);
    fprintf(stderr,"Schalter: -h     fuer diese Hilfe\n");
    fprintf(stderr,"Zum Format der Ein-/Ausgabe siehe Beispieldateien\n");
    fprintf(stderr,"\nMirko Belickin und Joachim Wagner - 22. November 1999\n");
    exit(fehler);
  }
  if (argc>1) {                         
    if (!(temp=fopen(argv[1],"rb"))) {
      fprintf(stderr,"Oeffnen der Datei %s fehlgeschlagen.", argv[1]);
      exit(1);
    }
    in = temp;
  }
  if (argc>2) {
    if (!(temp=fopen(argv[2],"wb"))) {
      fprintf(stderr,"Oeffnen der Datei %s fehlgeschlagen.", argv[2]);
      exit(1);
    }
    out = temp;
  }
  polynom = allg_interpolation.aktion->laden(in, NULL);

  c = polynom->aktion->imVektor(polynom, 2); /* Koeffizientenvektor auslesen */
  c->aktion->speichern(c, out);              /* und speichern                */

  if ( ferror(in) ) {
    fprintf(stderr, "Fehler in der Eingabe.\n");
    retcode = 1;
  }else{
    retcode = 0;
  }  
  if (in  != stdin)  fclose(in);                /* Dateien ggf. schliessen */
  if (out != stdout) fclose(out);

  return retcode;
}
