/* matrix.h
 * erstellt: Die Okt 26 16:21:25 CEST 1999
 * Autoren : Mirko Belickin und Joachim Wagner
 */

#ifndef __MATRIX_H
#define __MATRIX_H

#include <stdio.h>

typedef struct matrixFunktionen matrixFunktionen;

typedef struct matrix {
    matrixFunktionen *aktion;
    void *daten;
} matrix;


/* Makros, die die Benutzung vereinfachen */

#define AKTION3(m,a,x1,x2,x3)    (m)->aktion->a##((m),x1,x2,x3)
#define AKTION2(m,a,x1,x2)       (m)->aktion->a##((m),x1,x2)
#define AKTION1(m,a,x1)          (m)->aktion->a##((m),x1)
#define AKTION0(m,a)             (m)->aktion->a##(m)


struct matrixFunktionen {
    int (* ausgeben)(matrix *m, FILE *datei);       /* graphisch */
    int (* speichern)(matrix *m, FILE *datei);      /* strukturell */
    matrix  * (* laden)(FILE *datei, void *vater);
    int (* freigeben)(matrix *m, void *vater);
    matrix  * (* kreieren)(void *schablone, void *vater);
    double (* amIndex)(matrix *m, int zeile, int spalte);
    void (* setzen)(matrix *m, int zeile, int spalte, double d); 
    int (* breite)(matrix *m);
    int (* hoehe)(matrix *m);
    char * (* name)(matrix *m);
    matrix * (* klasse)(matrix *m);
    /* folgende Funktionen muessen nicht fuer jede Matrix definiert werden: */
    matrix * (* produkt)(matrix *a, matrix *b);
    matrix * (* imVektor)(matrix *m, int vektor);
    int (* bandbreite)(matrix *m);
};


/* nuetzliche Funktionen (fuer allgemeine Matrizen) */
/* ================================================ */


/* matrix_kopfzeile
 * liest die Kopfzeile eine Matrix ein, ueberprueft den Typ der Matrix
 * und gibt den Namen der Matrix als lokalen String zureuck, den sich
 * der Aufrufer zur weiteren Verwendung kopieren muss.
 * Wird ein leerer String als matrixtyp uebergeben, so wird der Typ er-
 * mittelt und in matrixtyp abgelegt. Ein nachfolgender Aufruf dieser
 * Funktion bearbeitet dann keine neue Zeile sondern liefert den Namen
 * der Matrix, deren Typ zuvor bestimmt wurde.
 */


char *matrix_kopfzeile(FILE *datei, char *matrixtyp);


/* weiteres */

typedef struct vektor2d {
  int x,y;
} vektor2d;


/* Rechengenauigkeit */

#define EPSILON (pow(2,-120))

/* EPSILON kann noch durch 64 geteilt werden, ohne subnormal zu werden (volle
 * 52Bit Nachkomma) und bleibt selbst nach Teilen durch 10^17 echt groesser 0.
 */

/* allgemeine Matrix-Funktionen (Aufruf ueber objekt->aktion->...) */

int matrix_ausgeben(matrix *m, FILE *datei);
matrix * matrix_klasse(matrix *m); /* liefert Art der Matrix */

#endif

