/* wertetab.c
 * Eingabe:  n + 1 Stuetzpunkte (Format siehe Beispieldateien)
 * Ausgabe:  Funktionswertetabelle zum Interpolationspolynom
 * erstellt: Die Nov 23 16:43:51 CET 1999
 * Autoren : Mirko Belickin und Joachim Wagner
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "funktion.h"
#include "matrix.h"
#include "nmmatrix.h"
#include "interplt.h"
#include "newtnipl.h"

extern interpolation allg_interpolation;
extern matrix nm_matrix;

extern funktionsbeispiel funktion[];
extern const int funktion_anzahl;


/* Hauptprogramm 
 *
 * erzeugt Beispiele
 */

int main(int argc, char *argv[])
{
  FILE *in = stdin;
  FILE *out = stdout;
  FILE *temp;
  int hilfe = 0;        /* 1 = Hilfe fuer Benutzer anzeigen */   
  int fehler = 0;       /* 1 = Programm falsch aufgerufen */
  int schalter = 1;     /* 0 = es folgen keine Schalter ('--' gefunden) */
  int i, n, m;
  double a, b;          /* Grenzen des Intervalls */
  double stelle;        /* aktuelle Stelle der Funktionswertberechnung */
  char tempname[140];
  matrix *y, *x;
  nmmatrixschablone schablone;
  interpolation *polynom;
  
  while ((schalter) && (argc>1) && (*argv[1]=='-')) {/* Schalter bearbeiten */
    switch ((argv[1])[1]) {
      case 'h': hilfe = 1;
                break;
      case '-': schalter = 0;
                break;
      default:  fprintf(stderr,"Schalter %s unbekannt.\n", argv[1]);
                fehler = 1;
                break;
    }
    argv[1] = argv[0];
    argv++; 
    argc--;
  }
  
  if ((argc>4) || (argc<2)) fehler = 1;
  if ((hilfe) || (fehler)) {             /* Hilfetext anzeigen */
    fprintf(stderr,"\nBenutzung: "
                   "%s [Schalter] [--] n [Eingabedatei [Ausgabedatei]]\n\n",
                   argv[0]);
    fprintf(stderr,"Schalter: -h fuer diese Hilfe\n");
    fprintf(stderr," n+1 = Anzahl der Werte\n");
    fprintf(stderr,"\nMirko Belickin und Joachim Wagner - 23. November 1999\n");
    exit(fehler);
  }
  if (argc>1) {
    n = atoi(argv[1]);
    if (n<1) {
      fprintf(stderr, "n muss mindestens 1 sein!\n");
      exit(1);
    }
  }
  if (argc>2) {
    if (!(temp=fopen(argv[2],"rb"))) {
      fprintf(stderr,"Oeffnen der Datei %s fehlgeschlagen.", argv[2]);
      exit(1);
    }
    in = temp;
  }
  if (argc>3) {
    if (!(temp=fopen(argv[3],"wb"))) {
      fprintf(stderr,"Oeffnen der Datei %s fehlgeschlagen.", argv[3]);
      exit(1);
    }
    out = temp;
  }
  
  polynom = allg_interpolation.aktion->laden(in, NULL);

  schablone.n = n + 1;
  schablone.m = 1;
  schablone.init = 1;          /* mit 0 initialisieren */
  schablone.name = "X-Werte";
  x = nm_matrix.aktion->kreieren(&schablone, NULL);
  sprintf(tempname, "Y-Werte zum Polynom %s", 
          polynom->aktion->name(polynom));
  schablone.name = tempname;
  y = nm_matrix.aktion->kreieren(&schablone, NULL);
  a = polynom->aktion->linkeGrenze(polynom);
  b = polynom->aktion->rechteGrenze(polynom);
  if ((b - a) < EPSILON) {
      fprintf(stderr,"Intervall [%lf,%lf] numerisch zu klein oder "
                           "ungueltig.\n", a, b);
      exit(1);
  }
  for (i=0; i <= n; i++) {
    x->aktion->setzen(x, i+1, 1,             /* Stelle bestimmen und als  */
       stelle = (i * b + (n - i) * a) / n);  /* X-Wert eintragen          */
                  /* umgeformt, so dass numerisch x(i) > b nicht auftritt */
    y->aktion->setzen(y, i+1, 1,             /* Funktionswert in Y-Vektor */
       polynom->aktion->funktionswert(polynom, stelle));     /* eintragen */ 
  }
 
  x->aktion->speichern(x, out);              /* Vektoren speichern        */
  y->aktion->speichern(y, out);
  x->aktion->freigeben(x, NULL);
  y->aktion->freigeben(y, NULL);

  if (out != stdout) fclose(out);            /* Dateien ggf. schliessen   */
  if (in != stdin) fclose(in);               /* Dateien ggf. schliessen   */
  return 0;
}
