#!/bin/sh
#
# Gibt eine Tabelle (':'-separiert) mit den Abweichungen des Interpolations-
# polynoms von der zu approximierenden Funktion aus und erstellt jeweils
# Funktionsgraphen.

# 
# sicherstellen, dass alles bereit
#
(cd source && make install)
test -d ergebnis || mkdir ergebnis

for ART in a t         # a = aequidistant, t = Tschebyscheff-Polynom
do
  # 
  # Kopfzeile zu Fehlertabelle 
  #
  for GRAD in 2 4 7 10 15 30; do echo -n ":"; echo -n $GRAD; done
  echo

  # 
  # Schleife ueber die zu waehlende Funktion und den Grad des
  # Polynoms
  #
  for FNR in 1 2 3 4; do               # Nr. der Beispielfunktion
    echo -n $FNR
    for GRAD in 2 4 7 10 15 30; do     # Grad des Interpolationpolynoms

      PRUEF=ergebnis/${FNR}_${GRAD}.tab
      NAME=ergebnis/${FNR}${ART}${GRAD}

      #
      # Interpolationpolynom bestimmen
      #
      gbsp -$ART $GRAD $FNR ${NAME}.ipl

      #
      # Koeffizienten der Newton-Darstellung ausgeben
      #
      koeffizi ${NAME}.ipl ${NAME}.cof

      #
      # Wertetabellen erstellen
      #
      wertetab 1000 ${NAME}.ipl ${NAME}.tab
      prueftab 1000 ${FNR} $PRUEF

      #
      # Fehler bestimmen und in Tabelle eintragen
      #
      echo -n ":"
      echo -n `fehler $PRUEF ${NAME}.tab | cut -d' ' -f3` 
 
      #
      # Graphen erstellen
      #
      plot.sh $FNR $ART $GRAD ps ergebnis/ >/dev/null 2>/dev/null
      echo | gs -sDEVICE=tiffpack -r300 -sPAPERSIZE=a5 \
                 -sOutputFile=${NAME}.tif ${NAME}.ps -c quit \
            >/dev/null 2>/dev/null
      #gimp -n -b '' '(gimp-quit 0)'
      #mv temp2.tif ${NAME}.tif
      #rm temp.tif
     
    done

    #
    # Tabellenzeile abschliessen
    #
    echo     
  done

  #
  # und die naechste Tabelle
  #
  echo     
done
